/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.structure.apio.internal.util;

import com.liferay.apio.architect.functional.Try;
import com.liferay.dynamic.data.mapping.model.Value;

import java.util.Locale;
import java.util.function.BiFunction;
import java.util.function.Function;

/**
 * @author Paulo Cruz
 */
public final class LocalizedValueUtil {

	public static <T> BiFunction<T, Locale, String> getLocalizedString(
		Function<T, Value> function) {

		return (value, locale) -> Try.fromFallible(
			() -> function.apply(value)
		).map(
			localizedValue -> localizedValue.getString(locale)
		).orElse(
			null
		);
	}

}